﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//
// VSCoverageToEmma - Visual Studio Coverage file to Emma converter
// Copyright (c) Kouji Matsui, All rights reserved.
//
// * Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Data;
using System.Diagnostics;
using System.Reflection;

namespace VSCoverageToEmma.Converters
{
	internal sealed class CoverageInfoManagerAccessor
	{
		private readonly Type type_;
		private readonly MethodInfo createInfoFromFileMethod_;

		public CoverageInfoManagerAccessor(Type type)
		{
			Debug.Assert(type != null);
			Debug.Assert(type.IsClass == true);
			Debug.Assert(type.FullName == "Microsoft.VisualStudio.CodeCoverage.CoverageInfoManager");

			type_ = type;
			createInfoFromFileMethod_ = type_.GetMethod(
				"CreateInfoFromFile",
				BindingFlags.Public | BindingFlags.Static,
				null,
				new[] { typeof(string) },
				null);
			Debug.Assert(createInfoFromFileMethod_ != null);
		}

		public DataSet BuildDataSet(string exePath, string symPath, string vsCoveragePath)
		{
			lock (type_)
			{
				type_.GetProperty("ExePath", BindingFlags.Public | BindingFlags.Static).SetValue(null, exePath, null);
				type_.GetProperty("SymPath", BindingFlags.Public | BindingFlags.Static).SetValue(null, symPath, null);

				using (var coverageInfo = (IDisposable)createInfoFromFileMethod_.
					Invoke(null, new[] { vsCoveragePath }))
				{
					var coverageInfoType = coverageInfo.GetType();
					var buildDataSetMethod = coverageInfoType.GetMethod(
						"BuildDataSet",
						BindingFlags.Public | BindingFlags.Instance);
					Debug.Assert(buildDataSetMethod != null);

					return (DataSet)buildDataSetMethod.Invoke(coverageInfo, new[] { (Guid[])null });
				}
			}
		}
	}
}
