﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//
// VSCoverageToEmma - Visual Studio Coverage file to Emma converter
// Copyright (c) Kouji Matsui, All rights reserved.
//
// * Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

using Microsoft.Win32;
using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;

namespace VSCoverageToEmma.Converters
{
	public abstract class V2ConverterFactory : IVSCoverageConverterFactory
	{
		private readonly string name_;
		private readonly CoverageInfoAccessor accessor_;

		protected V2ConverterFactory(string name, string internalVersionString)
		{
			Debug.Assert(string.IsNullOrEmpty(name) == false);
			Debug.Assert(string.IsNullOrEmpty(internalVersionString) == false);

			name_ = name;

			var installDir = InstallationDetector.GetInstallDir(internalVersionString);
			if (installDir == null)
			{
				throw new FileNotFoundException(string.Format("{0} installation not found.", name_));
			}

			var privateAssemblyPath = Path.Combine(installDir, "PrivateAssemblies");
			var analysisPath = Path.Combine(privateAssemblyPath, "Microsoft.VisualStudio.Coverage.Analysis.dll");

			var assembly = Assembly.LoadFrom(analysisPath);
			var coverageInfoType = assembly.GetType("Microsoft.VisualStudio.Coverage.Analysis.CoverageInfo");

			accessor_ = new CoverageInfoAccessor(coverageInfoType);
		}

		public string Name
		{
			get
			{
				return name_;
			}
		}

		public IVSCoverageConverter CreateConverter(string binariesFolderPath, string symbolsFolderPath)
		{
			return new V2Converter(accessor_, binariesFolderPath, symbolsFolderPath);
		}
	}
}
